﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.Collections.ObjectModel;
using System.IO;

namespace ThinkGeo_PanningBug
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            winformsMap1.MapUnit = GeographyUnit.Meter;

            ManagedProj4Projection proj = new ManagedProj4Projection();
            proj.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.White);
            graticuleAdornmentLayer.GraticuleTextFont = new GeoFont("Arial", 8, DrawingFontStyles.Bold);
            graticuleAdornmentLayer.Projection = proj;

            winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

            // Setup the culture overlay
            InMemoryFeatureLayer cultureLayer = new InMemoryFeatureLayer();
            cultureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.FromArgb(100, GeoColor.SimpleColors.Green);
            cultureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay cultureOverlay = new LayerOverlay();
            cultureOverlay.Layers.Add("CultureLayer", cultureLayer);
            cultureOverlay.Layers["CultureLayer"].Name = "CultureLayer";
            winformsMap1.Overlays.Add("CultureOverlay", cultureOverlay);
            winformsMap1.Overlays["CultureOverlay"].Name = "CultureOverlay";

            Collection<string> baseFeatures = new Collection<string>();
            baseFeatures.Add(Application.StartupPath + @"\Samples\ShapeFiles\Countries02.shp");
            baseFeatures.Add(Application.StartupPath + @"\Samples\ShapeFiles\UTM.shp");
            AddSystemShapeFileFeatureLayers(baseFeatures, cultureOverlay, winformsMap1);

            RectangleShape rect = new RectangleShape(-80, 50, 80, -50);
            proj.Open();
            rect = proj.ConvertToExternalProjection(rect);
            winformsMap1.CurrentExtent = rect;

            winformsMap1.Refresh();
        }

        private void button1_Click(object sender, EventArgs e)
        {

        }

        public static void AddSystemShapeFileFeatureLayers(IEnumerable<string> fileNames, LayerOverlay layerOverlay,
            WinformsMap winformsMap)
        {
            foreach (string fileName in fileNames)
            {
                try
                {
                    string layerName = new FileInfo(fileName).Name;
                    if (!layerOverlay.Layers.Contains(layerName))
                    {
                        string indexFile = fileName.Substring(0, fileName.Length - 4) + ".idx";
                        if (!File.Exists(indexFile))
                        {
                            ShapeFileFeatureLayer.BuildIndexFile(fileName, BuildIndexMode.DoNotRebuild);
                        }

                        ShapeFileFeatureLayer layer = new ShapeFileFeatureLayer(fileName);
                        layer.Encoding = Encoding.ASCII;
                        layer.Open();
                        layer.Name = layerName;
                        layer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen = new GeoPen(GeoColor.SimpleColors.Black, 1);
                        layer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(PointSymbolType.Circle, new GeoSolidBrush(GeoColor.SimpleColors.Black), 9);
                        layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen = new GeoPen(GeoColor.SimpleColors.Black, 1);
                        layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                        // Now, look into the prj file and determine what the settings are
                        Proj4Projection proj4 = new Proj4Projection();

                        // get the text of the .prj file if it exists.
                        string prjFileName = fileName.Substring(0, fileName.Length - 4) + ".prj";
                        if (File.Exists(prjFileName))
                        {
                            string prjFileText = System.IO.File.ReadAllText(prjFileName);

                            string[] varProjection = prjFileText.Split(new Char[] { ',' });
                            int startByte = varProjection[0].IndexOf("[") + 1;
                            string datumType = varProjection[0].ToString().Substring(startByte);
                            //textProj.Text = "Input prj values: " + prjFileText;
                            // set the internal projection.  The new ConvertPrjToProj4 method is the key to this process.

                            proj4.InternalProjectionParametersString = Proj4Projection.ConvertPrjToProj4(prjFileText);
                            // set the external projection.

                            // ##################################
                            // Spherical Mercator 18th June, 2012
                            //proj4.ExternalProjectionParametersString = Proj4Projection.GetWgs84ParametersString();  //WGS84 (Geodetic) 
                            proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
                            // ##################################

                            layer.FeatureSource.Projection = proj4;
                            proj4.Open();
                            RectangleShape bBox = layer.GetBoundingBox();
                            //proj4.Close();

                        }
                        else
                        {
                        }

                        //layerOverlay.Lock.EnterWriteLock();
                        try
                        {
                            layer.Name = layerName;
                            layer.Validate();
                            layerOverlay.Layers.Add(layerName, layer);

                            // Build an index file if it doesn't exist
                            ShapeFileFeatureLayer.BuildIndexFile(fileName, BuildIndexMode.DoNotRebuild);

                        }
                        finally
                        {
                            //layerOverlay.Lock.ExitWriteLock();
                            layer.Close();
                        }
                    }
                }
                catch (FileLoadException fle)
                {
                    MessageBox.Show(fle.Message, fle.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, (MessageBoxOptions)0);
                }
                catch (InvalidOperationException ex)
                {
                    MessageBox.Show(ex.Message, ex.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, (MessageBoxOptions)0);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, ex.GetType().ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, (MessageBoxOptions)0);
                }
            }
        }

    }
}
